from decimal import Decimal
from django.template.loader import get_template
from django.utils.translation import gettext_lazy as _

def template_renderer(template_name, **kwargs):
	'''
	A utility fonction that return a renderer function that renders a template by passing it the context.
	Optional keywords can be specified to be added to the context before rendering the template.
	If the value of the keyword is callable, it will be called with the boundfield as parameter.
	Examples:
	template_renderer('path/to/my_template.html', color = 'red')
	This will add the item ('color': 'red') to the context before rendering the template
	
	template_renderer('path/to/my_template.html', color = lambda b: 'red' if b.value() < 0 else 'blue')
	This will add the item ('color': 'red') to the context if the value of the boundfield is negative,
	else it will add the the item ('color': 'blue') before rendering the template.
	'''
	def render_template(context):
		template = get_template(template_name)
		boundfield = context['boundfield']
		for key, value in kwargs.items():
			if callable(value):
				context[key] = value(boundfield)
			else:
				context[key] = value
		return template.render(context)
	return render_template



from django import forms
from django.template.loader import get_template
from ..registry import RendererRegistry

# Use the field.html template render method as the renderer function
renderer = get_template('default/field.html').render

# Create the registry with all Django built-in form fields
renderer_registry = RendererRegistry({
	forms.BooleanField: renderer,
	forms.CharField: renderer,
	forms.ChoiceField: renderer,
	forms.TypedChoiceField: renderer,
	forms.DateField: renderer,
	forms.DateTimeField: renderer,
	forms.DecimalField: renderer,
	forms.DurationField: renderer,
	forms.EmailField: renderer,
	forms.FileField: renderer,
	forms.FilePathField: renderer,
	forms.FloatField: renderer,
	forms.ImageField: renderer,
	forms.IntegerField: renderer,
	forms.GenericIPAddressField: renderer,
	forms.MultipleChoiceField: renderer,
	forms.TypedMultipleChoiceField: renderer,
	forms.NullBooleanField: renderer,
	forms.RegexField: renderer,
	forms.SlugField: renderer,
	forms.TimeField: renderer,
	forms.URLField: renderer,
	forms.UUIDField: renderer,
	forms.ComboField: renderer,
	forms.SplitDateTimeField: renderer,
	forms.ModelChoiceField: renderer,
	forms.ModelMultipleChoiceField: renderer,
})

print('evaluated default')



import importlib
from django import template
from django.conf import settings

register = template.Library()

@register.simple_tag
def renderer(boundfield, registry = None, **kwargs):
	'''
	Render a boundfield to HTML using the specified renderer registry
	
	The context dict contains the following key/values:
		form: The form the field belongs to
		boundfield: The boundfield itself
		html_name: The HTML name attribute for the field
		value: The current value of the field
		label: The label of the field
		id: The id for the label of the field
		help_text: The help_text of the field
		errors: The list of Validation errors for the field
		disabled: True if the field is disabled
		is_hidden: True if the field is hidden
		required: True if the field is required
	'''
	
	# Get the registry
	if registry is None:
		registry = settings.getattr('DEFAULT_BOUNDFIELD_RENDERER_REGISTRY', 'boundfield_renderer.registries.default')
	
	# If the registry is a path, resolve it
	if isinstance(registry, str):
		try:
			module_name, registry_name = registry.rsplit('.', 1)
			module = importlib.import_module(module_name)
			registry = getattr(module, registry_name)
		except Exception as why:
			raise ValueError('Could not find registry "%s"' % registry) from why
	
	# Get the approriate renderer
	renderer = registry[type(boundfield.field)]
	
	# Create the context dict
	context = {
		'form': boundfield.form,
		'boundfield': boundfield,
		'html_name': boundfield.html_name,
		'value': boundfield.value(),
		'label': boundfield.label,
		'id': boundfield.id_for_label,
		'help_text': boundfield.help_text,
		'errors': boundfield.errors,
		'disabled': boundfield.field.disabled,
		'is_hidden': boundfield.is_hidden,
		'required': boundfield.field.required
	}
	
	# Add the extra context kwargs
	context.update(kwargs)
	
	return renderer(context)



import inspect
from collections import UserDict

class RendererRegistry(UserDict):
	'''
	Registry for the form field renderers.
	
	A renderer is a function that takes a context dict and returns the html
	representation of the form field, for example the render method of a
	Template instance.
	
	'''
	
	def __getitem__(self, key):
		'''Return the best renderer function for the field'''
		
		# Look up the chain of inheritance for a registered renderer
		for form_class in inspect.getmro(key):
			try:
				return super().__getitem__(form_class)
			except KeyError:
				pass
		
		# If no renderer was found raise an Exception
		raise ValueError('No renderer found for field %s' % key)
	
	def register(self, renderer):
		'''
		Form Field class decorator to register a renderer into the registry.
		
		Example:
		from django.template.loader import get_template
		from my_registry import renderer_registry
		
		renderer_registry.register(get_template('/path/to/my_field.html').render)
		class MyField(forms.Field):
			pass
		'''
		
		def registerer(form_field_class):
			self[form_field_class] = renderer
			return form_field_class
		
		return registerer



from django import forms
from ..registry import RendererRegistry
from ..utils import template_renderer

# Helper functions
def get_widget(attrs = None):
	'''Return the rendered widget with extra attrs'''
	def render_widget(boundfield):
		return boundfield.as_widget(attrs = attrs)
	return render_widget

# Create the registry with all Django built-in form fields
renderer_registry = RendererRegistry({
	forms.BooleanField: template_renderer('bootstrap4/form-check.html', widget=get_widget({'class': 'form-check-input'})),
	forms.CharField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.ChoiceField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.TypedChoiceField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.DateField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.DateTimeField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.DecimalField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.DurationField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.EmailField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.FileField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control-file'})),
	forms.FilePathField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.FloatField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.ImageField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control-file'})),
	forms.IntegerField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.GenericIPAddressField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.MultipleChoiceField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.TypedMultipleChoiceField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.NullBooleanField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.RegexField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.SlugField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.TimeField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.URLField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.UUIDField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.ComboField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.SplitDateTimeField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.ModelChoiceField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
	forms.ModelMultipleChoiceField: template_renderer('bootstrap4/form-group.html', widget=get_widget({'class': 'form-control'})),
})
